<div ng-controller="SlideItemListCtrl" class="content scrollcontent">
  <h1 ng-if="!element.id" translate>Agenda</h1>
  <h1 ng-if="element.id">{{ rootItem.getTitle() }}</h1>

  <div class="agendalist">
    <!-- item type: AGENDA_ITEM = 1, HIDDEN_ITEM = 2 -->
    <p ng-repeat="item in items | filter: {type: 1}"
        ng-class="{mainitem: item.parentCount == 0, subitem: item.parentCount != 0, done: item.closed}">
      <span ng-repeat="n in [].constructor(item.parentCount) track by $index">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</span>
      {{ item.getListViewTitle() }}
    </p>
  </div>
</div>
