<h1><translate>Vote</translate> {{ voteNumber }}</h1>

<uib-alert ng-show="alert.show" type="{{ alert.type }}" ng-click="alert={}" close="alert={}">
  {{ alert.msg }}
</uib-alert>

<p>
  <translate>Special values</translate>:
  <span class="badge badge-success">-1</span> = <translate>majority</translate>
  <span class="badge">-2</span> = <translate>undocumented</translate>

<form name="motionPollForm" ng-submit="save(model)">
  <formly-form model="model" fields="formFields">
    <button type="submit" ng-disabled="motionPollForm.$invalid" class="btn btn-primary" translate>
      Save
    </button>
    <button ng-click="closeThisDialog()" class="btn btn-default" translate>
      Cancel
    </button>
  </formly-form>
</form>
