<div class="header">
  <div class="title">
    <h1 translate>Legal notice</h1>
  </div>
</div>

<div class="details">
  <div ng-bind-html="config('general_event_legal_notice')"></div>
  <hr>
  <p>OpenSlides {{ core_version }}
  <div ng-show="plugins.length">
    <p translate>Installed plugins:</p>
    <ol>
      <li ng-repeat="plugin in plugins">
        {{ plugin.verbose_name }}: {{ plugin.version }}
    </ol>
  </div>
</div>
