<div class="header">
  <div class="title">
    <h1 translate>Change password</h1>
  </div>
</div>

<div class="details">
  <p ng-if='formError' class="text-danger">
    <strong>{{ formError }}</strong>
  </p>
  <form name="userForm" >
    <div class="form-group">
      <label for="inputOldPassword" translate>Old password</label>
      <input type="password"
             ng-model="oldPassword"
             class="form-control"
             name="inputOldPassword"
             required>
    </div>
    <div class="form-group">
      <label for="inputNewPassword" translate>New password</label>
      <input type="password"
             ng-model="newPassword"
             class="form-control"
             name="inputNewPassword"
             required>
    </div>
    <div class="form-group">
      <label for="inputNewPassword2" translate>Confirm new password</label>
      <input type="password"
             ng-model="newPassword2"
             class="form-control"
             name="inputNewPassword2"
             required>
    </div>
    <button type="submit" ng-click="save(user)" class="btn btn-primary" translate>
      Save
    </button>
    <button ui-sref="users.user.list" class="btn btn-default" translate>
      Cancel
    </button>
  </form>
</div>
