<div class="header">
  <div class="title">
    <div class="submenu">
      <a ui-sref="users.group.list" class="btn btn-sm btn-default">
        <i class="fa fa-angle-double-left fa-lg"></i>
        <translate>Back to overview</translate>
      </a>
    </div>
    <h1>{{ group.name | translate }}</h1>
    <h2 translate>Group</h2>
  </div>
</div>

<div class="details">
  <p class="lead" translate>Permissions:</p>
  <ul ng-repeat="permission in groupPermissionNames">
    <li>{{ permission | translate }}
  </ul>
</div>
