<form ng-submit="login(username, password)">
  <div class="modal-header">
    <img src="/static/img/openslides-logo.png" alt="OpenSlides" class="login-logo center-block">
  </div>
  <div class="modal-body loginForm">
    <uib-alert ng-repeat="alert in alerts" type="{{ alert.type }}" close="closeAlert($index)">
      <span ng-bind-html="alert.msg"><span>
    </uib-alert>
    <div class="input-group form-group">
      <div class="input-group-addon"><i class="fa fa-user"></i></div>
      <input os-focus-me type="text" ng-model="username" class="form-control input-lg"
          placeholder="{{ 'Username' | translate }}">
    </div>
    <div class="input-group form-group">
      <div class="input-group-addon"><i class="fa fa-key"></i></div>
      <input type="password" ng-model="password" class="form-control input-lg"
          placeholder="{{ 'Password' | translate }}">
    </div>
    <div class="form-group">
      <button type="submit" class="btn btn-primary" translate>
        Login
      </button>
      <button ng-if="guestAllowed" ng-click="guestLogin()" class="btn btn-default" translate>
        Continue as guest
      </button>
    </div>
  </div>
</form>
