<div ng-controller="SlideMotionCtrl" class="content scrollcontent">

  <div id="sidebox">
    <!-- State -->
    <h3 translate>State</h3>
    {{ motion.state.name | translate }}

    <!-- Submitters -->
    <h3 ng-if="motion.submitters.length > 0" translate>Submitters</h3>
    <div ng-repeat="submitter in motion.submitters">
      {{ submitter.get_full_name() }}<br>
    </div>

    <!-- Poll results -->
    <h3 ng-if="motion.polls.length > 0" translate>Voting result</h3>
      <div ng-repeat="poll in motion.polls | orderBy:'-id'" class="spacer">
        <div ng-show="poll.has_votes" class="pollresults">
          <strong ng-if="motion.polls.length > 1">
            <translate>Vote</translate> {{ motion.polls.length - $index }}:
          </strong>
          <table class="transparentTable spacer">
            <!-- yes -->
            <tr>
              <td class="icon">
                <i class="fa fa-thumbs-up fa-2x"></i>
              <td ng-init="voteYes = poll.getVote(poll.yes)">
                <span class="result_label"><translate>Yes</translate>:</span>
                <span class="result_value">
                  {{ voteYes.value }} {{ voteYes.percentStr }}
                </span>
                <div ng-if="voteYes.percentNumber">
                  <uib-progressbar value="voteYes.percentNumber" type="success"></uib-progressbar>
                </div>
            <!-- no -->
            <tr>
              <td class="icon">
                <i class="fa fa-thumbs-down fa-2x"></i>
              <td ng-init="voteNo = poll.getVote(poll.no)">
                <span class="result_label"><translate>No</translate>:</span>
                <span class="result_value" >
                  {{ voteNo.value }} {{ voteNo.percentStr }}
                </span>
                <div ng-if="voteNo.percentNumber">
                  <uib-progressbar value="voteNo.percentNumber" type="danger"></uib-progressbar>
                </div>
            <!-- abstain -->
            <tr>
              <td class="icon">
                <strong style="font-size: 26px">&empty;</strong>
              <td ng-init="voteAbstain = poll.getVote(poll.abstain)">
                <span class="result_label"><translate>Abstain</translate>:</span>
                <span class="result_value">
                  {{ voteAbstain.value }} {{ voteAbstain.percentStr }}
                </span>
                <div ng-if="voteAbstain.percentNumber">
                  <uib-progressbar value="voteAbstain.percentNumber" type="warning"></uib-progressbar>
                </div>
          </table>
        </div>
      </div>
  </div>

  <!-- Title -->
  <div id="title">
    <h1>{{  motion.agenda_item.getTitle() || motion.getTitle() }}</h1>
    <h2>
      <translate>Motion</translate> {{ motion.identifier }}
      <span ng-if="motion.versions.length > 1" >| Version {{ motion.getVersion().version_number }}</span>
    </h2>
  </div>

  <!-- Text -->
  <div ng-bind-html="motion.getText() | trusted"></div>

  <!-- Reason -->
  <h3 ng-if="motion.getReason()" translate>Reason</h3>
  <div ng-bind-html="motion.getReason() | trusted"></div>
</div>
