<!-- custom angular formly template for angular-chosen multiple form field -->
<select multiple chosen
    data-ng-model="model[options.key]"
    ng-required="{{ to.required }}"
    ng-options="{{ to.ngOptions }}"
    search-contains="true"
    placeholder-text-multiple="'{{ to.placeholder }}'"
    no-results-text="'No results available ...' | translate"
    class="form-control">
</select>
