<div class="header">
  <div class="title">
    <div class="submenu">
      <a ui-sref="motions.category.list" class="btn btn-sm btn-default">
        <i class="fa fa-angle-double-left fa-lg"></i>
        <translate>Back to overview</translate>
      </a>
    </div>
    <h1>{{ category.name }}</h1>
    <h2 translate>Category</h2>
  </div>
</div>

<div class="details">
  <strong translate>Prefix:</strong>
  {{ category.prefix }}
</div>
