<div class="header">
  <div class="title">
    <h1 translate>Search results</h1>
  </div>
</div>

<div class="details">
    <form class="input-group" ng-submit="search(query)">
      <input type="text" ng-model="query" class="form-control">
      <span class="input-group-btn">
        <button type="submit" class="btn btn-default" translate>Search</button>
      </span>
    </form>

  <div class="searchresults spacer-top-lg">
    <ol ng-show="results">
      <li ng-repeat="result in results">
        <a ng-if="!result.mediafileUrl" ui-sref="{{ result.urlState }}({{ result.urlParam }})">
          {{ result.getSearchResultName() }}
        </a>
        <a ng-if="result.mediafileUrl" href="{{ result.mediafileUrl }}" target="_blank">
          {{ result.getSearchResultName() }}
        </a>
        <br>
        <span class="grey">{{ result.getSearchResultSubtitle() | translate }}</span>
    </ol>
    <p ng-show="!results" translate>No results.</p>
  </div>
</div>
