<div ng-controller="SlideCountdownCtrl">
  <div ng-if="visible">
    <div class="countdown well pull-right"
        ng-class="{
          'negative': seconds <= 0,
          'warning': seconds <= config('agenda_countdown_warning_time') && seconds > 0 }">
      {{ seconds | osSecondsToTime}}
      <div class="description">{{ description }}</div>
    </div>
  </div>
</div>
