<div class="header">
  <div class="title">
    <h1 translate>Edit profile</h1>
  </div>
</div>

<div class="details">
  <p ng-if='formError' class="text-danger">
    <strong>{{ formError }}</strong>
  </p>
  <form name="userForm" >
    <div class="form-group">
      <label for="inputUsername" translate>Username</label>
      <input type="text"
             ng-model="user.username"
             class="form-control"
             name="inputUsername"
             required>
    </div>
    <div class="form-group row">
      <div class="col-xs-2">
        <label for="inputTitle" translate-comment="academic degree" translate>Title</label>
        <input type="text" ng-model="user.title" class="form-control" name="inputTitle">
      </div>
      <div class="col-xs-5">
        <label for="inputFirstName" translate>First name</label>
        <input type="text" ng-model="user.first_name" class="form-control" name="inputFirstName">
      </div>
      <div class="col-xs-5">
        <label for="inputLastName" translate>Last name</label>
        <input type="text" ng-model="user.last_name" class="form-control" name="inputLastName">
      </div>
    </div>
    <div class="form-group">
      <label for="inputStructureLevel" translate>Structure level</label>
      <input type="text" ng-model="user.structure_level" class="form-control" name="inputStructureLevel">
    </div>
    <div class="form-group">
      <label for="textAbout" translate>About me</label>
      <textarea ng-model="user.about_me" ui-tinymce="tinymceOption" class="form-control" name="textAbout" />
    </div>

    <button type="submit" ng-click="save(user)" class="btn btn-primary" translate>
      Save
    </button>
    <button ui-sref="users.user.list" class="btn btn-default" translate>
      Cancel
    </button>
  </form>
</div>
