<div class="header">
  <div class="title">
    <div class="submenu">
      <a ui-sref="agenda.item.list" class="btn btn-sm btn-default">
        <i class="fa fa-angle-double-left fa-lg"></i>
        <translate>Back to overview</translate>
      </a>
      <!-- List of speakers -->
      <a ui-sref="agenda.item.detail({id: customslide.agenda_item_id})" class="btn btn-sm btn-default">
        <i class="fa fa-microphone fa-lg"></i>
        <translate>List of speakers</translate>
      </a>
      <!-- project -->
      <a os-perms="core.can_manage_projector" class="btn btn-default btn-sm"
          ng-class="{ 'btn-primary': customslide.isProjected() }"
          ng-click="customslide.project()"
          title="{{ 'Project agenda item' | translate }}">
        <i class="fa fa-video-camera"></i>
      </a>
      <!-- edit -->
      <a os-perms="agenda.can_manage" ng-click="openDialog(customslide)"
          class="btn btn-default btn-sm"
          title="{{ 'Edit' | translate}}">
        <i class="fa fa-pencil"></i>
      </a>
    </div>
    <h1>{{ customslide.agenda_item.getTitle() }}</h1>
    <h2 translate>Agenda item</h2>
  </div>
</div>

<div class="details">
  <div ng-bind-html="customslide.text | trusted"></div>
  <h3 ng-if="customslide.attachments.length > 0" translate>Attachments</h3>
  <ul>
    <li ng-repeat="attachment in customslide.attachments">
      <a href="{{ attachment.mediafileUrl }}" target="_blank">
        {{ attachment.title_or_filename }}
      </a>
  </ul>
</div>
