<div class="header">
  <div class="title">
    <div class="submenu">
      <a ui-sref="users.group.list" class="btn btn-sm btn-default">
        <i class="fa fa-angle-double-left fa-lg"></i>
        <translate>Back to overview</translate>
      </a>
    </div>
    <h1 ng-if="group.id" translate>Edit group</h1>
    <h1 ng-if="!group.id" translate>New group</h1>
  </div>
</div>

<div class="details">
  <form name="groupForm">
    <div class="form-group">
      <label for="inputName" translate>Name</label>
      <input type="text" ng-model="group.name" class="form-control" name="inputName" required>
    </div>
    <div class="form-group">
      <label for="selectPermissions" translate>Permissions</label>
      <select multiple size="15" ng-model="group.permissions" class="form-control" id="selectPermissions">
          <option value="{{ permission.value }}" ng-repeat="permission in permissions">{{ permission.display_name | translate }}</option>
      </select>
    </div>

    <button type="submit" ng-click="save(group)" class="btn btn-primary" translate>
      Save
    </button>
    <button ui-sref="users.group.list" class="btn btn-default" translate>
      Cancel
    </button>
  </form>
</div>
