<div ng-controller="SlideMediafileCtrl" class="content fullscreen">
    <ng-pdf template-url="/static/templates/mediafiles/slide_mediafile_partial.html"
        ng-attr-scale="{{ element.scale }}"
        ng-attr-page="{{ element.page }}">
    </ng-pdf>
</div>
