/*
 * OpenSlides default projector styles
 *
 */

body{
    font-size: 20px !important;
    line-height: 24px !important;
    overflow: hidden;
    font-family: "Open Sans",Helvetica,Arial,sans-serif;
    color: #222;
}

/*** HEADER ***/
#header {
    box-shadow: 0 0 7px rgba(0,0,0,0.6);
    height: 70px;
    margin-bottom: 20px;
    font-family: 'Roboto Condensed Light';
}
#logo {
    position: relative;
    left: 50px;
    top: 10px;
    height: 50px;
    margin-right: 25px;
    float: left;
}
#eventdata {
    position: relative;
    left: 50px;
    top: 12px;
    height: 50px;
    overflow: hidden;
}
#eventdata .title {
    font-size: 26px;
    font-weight: bold;
 }
#eventdata .title.titleonly {
    position: relative;
    top: 12px;
}
#eventdata .description {
    font-size: 18px;
    opacity: 0.8;
}
#currentTime {
    font-family: 'Roboto Condensed Light';
    border: 0 solid #000000;
    font-size: 24px;
    position: absolute;
    text-align: right;
    top: 23px;
    right: 50px;
    padding-left: 30px;
}
#footer {
    font-family: 'Roboto Condensed Light';
    position: fixed;
    bottom: 0;
    height: 35px;
    width: 100%;
    font-size: 16px;
    padding-left: 50px;
    padding-right: 50px;
    padding-top: 5px;
    overflow: hidden;
    text-align: right;
}
#footer span {
    opacity: 0.8;
}

/*** CONTENT with base style elements ***/
.content {
    position: absolute;
    left: 50px;
    top: 150px;
    right: 50px;
    z-index: -1;
    line-height: 1.3em;
}
.scrollcontent {
    transition-property: margin, font-size;
    transition-duration: 1s;
}
.fullscreen {
    position: fixed;
    top: 0;
    left: 0;
    width: 100%;
    height: 100%;
    z-index: 1000;
    background-color: black;
}
.fullscreen canvas {
    margin: auto;
    display: block !important;
}

h1, h2, h3, h4, h5, h6 {
    font-family: "Roboto Condensed",Helvetica,Arial,sans-serif;
    font-weight: normal;
}

h1 {
    font-size: 2.25em;
    line-height: 1.1em;
}
h2 {
    font-size: 28px;
    margin-top: 15px;
    display: block;
    color: #9a9898;
}
h3 {
    color: #222;
    margin-bottom: 10px
}
#title {
    width: calc(100% - 230px);
    border-bottom: 5px solid #d3d3d3;
    margin-bottom: 40px;
}
#title h1 {
    margin-bottom: 0;
    padding-bottom: 0;
}
#title h2 {
    margin-top: 10px;
    margin-bottom: 5px;
}
#sidebox {
    width: 260px;
    float: right;
    margin: 44px -50px 20px 10px;
    right: 0;
    background: #d3d3d3;
    border-radius: 7px 0 0 7px;
    padding: 3px 7px 10px 19px;
}
#sidebox h3 {
    margin-top: 10px;
    margin-bottom: 0px;
}
ul, ol {
    margin: 0 0 10px 2em;
}
li {
    line-height: normal;
}
.result {
    line-height: 30px;
}
.result hr {
    border-top: 1px solid;
    margin: 5px 0;
    width: 10em;
}
.result.big {
    font-size: 120%;
    line-height: 40px;
}
.result .bold {
    font-weight: bold;
}
.electionresults table {
    width: calc(100% - 230px);
}
.electionresults .progress {
    margin-bottom: 0;
}
hr {
    margin: 10px 0;
}
.nobr {
    white-space: nowrap;
}
.white-space-pre-line {
    white-space: pre-line;
}


/*** Overlay ***/
.countdown {
    min-width: 260px;
    position: relative;
    margin: 0 0 10px 10px;
    top: 0px;
    right: 0px;
    padding: 23px 45px 0px 19px;
    min-height: 72px;
    font-size: 3.7em;
    font-weight: bold;
    text-align: right;
    border-radius: 7px 7px 7px 7px;
    z-index: 200;
    font-family: "Roboto Condensed",Helvetica,Arial,sans-serif;
}
.countdown .description {
    font-weight: normal;
    font-size: 18px;
    margin-top: 20px;
    padding-right: 6px;
}
.countdown.warning {
    color: #ed940d;
}
.countdown.negative {
    color: #CC0000;
}

.message_background {
    background-color: #777777;
    opacity: 0.8;
    position: absolute;
    top: 0;
    left: 0;
    width: 100%;
    height: 100%;
    z-index: 300;
}
.message {
    font-family: "Roboto Condensed",Helvetica,Arial,sans-serif;
    font-weight: normal;
    position: fixed;
    top: 35%;
    left: 10%;
    width: 80%;
    text-align: center;
    border-radius: 0.2em;
    background: #FFFFFF;
    font-size: 2.75em;
    padding: 0.2em 0;
    line-height: normal !important;
    z-index: 301;
}

/*** PDF presentation ***/
.rotate0 {
    transform: rotate(0deg);
}
.rotate90 {
    transform: rotate(90deg);
}
.rotate180 {
    transform: rotate(180deg);
}
.rotate270 {
    transform: rotate(270deg);
}

/*** Table style ***/
.table {
    border-collapse:collapse;
    border-color:#CCCCCC -moz-use-text-color #CCCCCC #CCCCCC;
    border-style:solid none solid solid;
    border-width:1px medium 1px 1px;
    margin:0;
    border-spacing:0px;
}
.table th {
    border-right:1px solid #CCCCCC;
    color:#333333;
    font-weight:normal;
    padding:10px 10px 10px 10px;
    text-align:left;
    text-transform:uppercase;
}
.table tr.odd td {
    background:none repeat scroll 0 0 #F1F1F1;
}
.table td {
    background:none repeat scroll 0 0 #F7F7F7;
    border-right:1px solid #CCCCCC;
    line-height:120%;
    padding: 10px 10px;
    vertical-align:middle;
}
tr.total td {
    border-top: 1px solid #333333;
    background-color: #e3e3e3;
}
tr.elected td {
    background-color: #BED4DE !important;
}
.transparentTable td {
    background-color: transparent;
    padding-right: 10px;
}
.pollresults table .icon {
    color: #636363;
    height: 45px;
}

/*** Agenda list ***/
.agendalist p {
    font-size: 140%;
}
.agendalist p.done {
    color: #9a9898;
}
.agendalist .mainitem {
    margin-top: 25px;
}
.agendalist .subitem {
    font-size: 100%;
    margin-bottom: 4px;
}
.spacer {
    margin-top: 10px;
}


/*** List of speakers ***/
.slimlabel.label {
    padding: 0px 10px;
}

.lastSpeakers {
    color: #9a9898;
    margin-left: 27px;
}
.currentSpeaker {
    font-weight: bold;
    margin-left: 0px;
}
.currentSpeaker i {
    padding-right: 5px;
}
.nextSpeakers {
    margin-left: 8px;
}
.nextSpeakers li {
    line-height: 150%;
}
