<div ng-controller="SlideListOfSpeakersCtrl" class="content scrollcontent">

  <!-- Title -->
  <div id="title">
    <h1>{{ item.getTitle() }}</h1>
    <h2>
      <span translate>List of speakers</span>
      <span ng-if="(item.speakers | filter: {begin_time: null}).length > 0">
      &ndash; {{ (item.speakers | filter: {begin_time: null}).length }} <translate>speakers</translate>
      <span ng-if="item.speaker_list_closed" class="slimlabel label label-danger" translate>Closed</span>
    </h2>
  </div>

  <!-- Last speakers -->
  <p ng-repeat="speaker in lastSpeakers = (item.speakers | filter: {end_time: '!!', begin_time: '!!'}) |
      limitTo: config('agenda_show_last_speakers') : (lastSpeakers.length - config('agenda_show_last_speakers'))" class="lastSpeakers">
      {{ speaker.user.get_full_name() }}

  <!-- Current speaker -->
  <p ng-repeat="speaker in item.speakers | filter: {end_time: null, begin_time: '!!'}"
      class="currentSpeaker">
      <i class="fa fa-microphone fa-lg"></i> {{ speaker.user.get_full_name() }}

  <!-- Next speakers -->
  <ol class="nextSpeakers">
    <li ng-repeat="speaker in item.speakers | filter: {begin_time: null}">
      {{ speaker.user.get_full_name() }}
  </ol>
</div>
